@@ -29,11 +29,11 @@ from codecs import lookup
 from imdb import IMDbBase
 from imdb.Person import Person
 from imdb.Movie import Movie
-from imdb._exceptions import IMDbDataAccessError
 from imdb.utils import analyze_title, build_title, analyze_name, \
                         build_name, canonicalTitle, canonicalName, \
                         normalizeName, normalizeTitle, re_titleRef, \
-                        re_nameRef, re_year_index, _articles
+                        re_nameRef, re_year_index, _articles, \
+                        analyze_company_name
 
 re_nameIndex = re.compile(r'\(([IVXLCDM]+)\)')
 
@@ -191,6 +191,8 @@ def nameVariations(name, fromPtdf=0):
 try:
     from cutils import ratcliff as _ratcliff
     def ratcliff(s1, s2, sm):
+        """Return the Ratcliff-Obershelp value between the two strings,
+        using the C implementation."""
         return _ratcliff(s1.encode('latin_1', 'replace'),
                         s2.encode('latin_1', 'replace'))
 except ImportError:
@@ -200,6 +202,7 @@ except ImportError:
                     ' data access systems will be slower.')
 
     def ratcliff(s1, s2, sm):
+        """Ratcliff-Obershelp similarity."""
         STRING_MAXLENDIFFER = 0.7
         s1len = len(s1)
         s2len = len(s2)
@@ -209,7 +212,6 @@ except ImportError:
             threshold = float(s2len) / s1len
         if threshold < STRING_MAXLENDIFFER:
             return 0.0
-        """Ratcliff-Obershelp similarity."""
         sm.set_seq2(s2.lower())
         return sm.ratio()
 
@@ -285,7 +287,7 @@ def scan_names(name_list, name1, name2, name3, results=0, ro_thresold=None,
 
 
 def scan_titles(titles_list, title1, title2, title3, results=0,
-                searchingEpisode=0, ro_thresold=None):
+                searchingEpisode=0, onlyEpisodes=0, ro_thresold=None):
     """Scan a list of titles, searching for best matches against
     the given variations."""
     if ro_thresold is not None: RO_THRESHOLD = ro_thresold
@@ -295,7 +297,6 @@ def scan_titles(titles_list, title1, title2, title3, results=0,
     sm3 = SequenceMatcher()
     sm1.set_seq1(title1.lower())
     sm2.set_seq2(title2.lower())
-    #searchingEpisode = 0
     if title3:
         sm3.set_seq1(title3.lower())
         if title3[-1] == '}': searchingEpisode = 1
@@ -303,6 +304,20 @@ def scan_titles(titles_list, title1, title2, title3, results=0,
     if title2 != title1: hasArt = 1
     resd = {}
     for i, t_data in titles_list:
+        if onlyEpisodes:
+            if t_data.get('kind') != 'episode':
+                continue
+            til = t_data['title']
+            if til[-1] == ')':
+                dateIdx = til.rfind('(')
+                if dateIdx != -1:
+                    til = til[:dateIdx].rstrip()
+            if not til:
+                continue
+            ratio = ratcliff(title1, til, sm1)
+            if ratio >= RO_THRESHOLD:
+                resd[i] = (ratio, (i, t_data))
+            continue
         if searchingEpisode:
             if t_data.get('kind') != 'episode': continue
         elif t_data.get('kind') == 'episode': continue
@@ -348,3 +363,40 @@ def scan_titles(titles_list, title1, title2, title3, results=0,
     return res
 
 
+def scan_company_names(name_list, name1, results=0, ro_thresold=None):
+    """Scan a list of company names, searching for best matches against
+    the given name.  Notice that this function takes a list of
+    strings, and not a list of dictionaries."""
+    if ro_thresold is not None: RO_THRESHOLD = ro_thresold
+    else: RO_THRESHOLD = 0.6
+    sm1 = SequenceMatcher()
+    sm1.set_seq1(name1.lower())
+    resd = {}
+    withoutCountry = not name1.endswith(']')
+    for i, n in name_list:
+        # XXX: on Symbian, here we get a str; not sure this is the
+        #      right place to fix it.
+        if isinstance(n, str):
+            n = unicode(n, 'latin1', 'ignore')
+        o_name = n
+        var = 0.0
+        if withoutCountry and n.endswith(']'):
+            cidx = n.rfind('[')
+            if cidx != -1:
+                n = n[:cidx].rstrip()
+                var = -0.05
+        # Distance with the company name.
+        ratio = ratcliff(name1, n, sm1) + var
+        if ratio >= RO_THRESHOLD:
+            if resd.has_key(i):
+                if ratio > resd[i][0]: resd[i] = (ratio,
+                                            (i, analyze_company_name(o_name)))
+            else:
+                resd[i] = (ratio, (i, analyze_company_name(o_name)))
+    res = resd.values()
+    res.sort()
+    res.reverse()
+    if results > 0: res[:] = res[:results]
+    return res
+
+
