@@ -26,11 +26,13 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """
 
 import sys
+import warnings
 from urllib import FancyURLopener, quote_plus
 from codecs import lookup
 
 from imdb import IMDbBase, imdbURL_movie_main, imdbURL_person_main, \
                 imdbURL_character_main, imdbURL_company_main, imdbURL_find
+from imdb.utils import analyze_title
 from imdb._exceptions import IMDbDataAccessError, IMDbParserError
 
 import searchMovieParser
@@ -43,14 +45,24 @@ import characterParser
 import companyParser
 
 
+class _FakeParser(object):
+    """Fake parser to replace obsoleted old parsers."""
+    def __init__(self, *args, **kwds):
+        pass
+
+    def parse(self, *args, **kwds):
+        return {'data': {}}
+
+
 class _ModuleProxy:
     """A proxy to instantiate and access parsers."""
     def __init__(self, module, defaultKeys=None, oldParsers=False,
-                useModule=None):
+                useModule=None, fallBackToNew=False):
         """Initialize a proxy for the given module; defaultKeys, if set,
         muste be a dictionary of values to set for instanced objects."""
         self.oldParsers = oldParsers
         self.useModule = useModule
+        self.fallBackToNew = fallBackToNew
         if defaultKeys is None:
             defaultKeys = {}
         self._defaultKeys = defaultKeys
@@ -66,7 +78,17 @@ class _ModuleProxy:
             kwds = {}
             if not self.oldParsers and self.useModule:
                 kwds = {'useModule': self.useModule}
-            obj = _entry[0][self.oldParsers](**kwds)
+            parserClass = _entry[0][self.oldParsers]
+            # Warns when the old parser was requested, but it's gone.
+            if self.oldParsers and parserClass is _entry[0][0]:
+                if not self.fallBackToNew:
+                    warnings.warn('Old parser "%s" not available; these ' \
+                                'data will not be available.' % name)
+                    parserClass = _FakeParser
+                else:
+                    warnings.warn('Old parser "%s" not available; falling ' \
+                                'back to the new one.' % name)
+            obj = parserClass(**kwds)
             attrsToSet = self._defaultKeys.copy()
             attrsToSet.update(_entry[1] or {})
             # Set attribute to the object.
@@ -179,7 +201,6 @@ class IMDbURLopener(FancyURLopener):
         if encode is None:
             encode = 'latin_1'
             # The detection of the encoding is error prone...
-            import warnings
             warnings.warn('Unable to detect the encoding of the retrieved '
                         'page [%s]; falling back to default latin1.' % encode)
         return unicode(content, encode, 'replace')
@@ -211,8 +232,8 @@ class IMDbHTTPAccessSystem(IMDbBase):
     accessSystem = 'http'
 
     def __init__(self, isThin=0, adultSearch=1, proxy=-1, oldParsers=False,
-                useModule=None, cookie_id=-1, cookie_uu=None,
-                *arguments, **keywords):
+                fallBackToNew=False, useModule=None, cookie_id=-1,
+                cookie_uu=None, *arguments, **keywords):
         """Initialize the access system."""
         IMDbBase.__init__(self, *arguments, **keywords)
         self.urlOpener =  IMDbURLopener()
@@ -238,24 +259,35 @@ class IMDbHTTPAccessSystem(IMDbBase):
                 self.set_cookies(cookie_id, cookie_uu)
         if proxy != -1:
             self.set_proxy(proxy)
+        if useModule is not None:
+            if not isinstance(useModule, (list, tuple)) and ',' in useModule:
+                useModule = useModule.split(',')
         _def = {'_modFunct': self._defModFunct, '_as': self.accessSystem}
         # Proxy objects.
         self.smProxy = _ModuleProxy(searchMovieParser, defaultKeys=_def,
-                                    oldParsers=oldParsers, useModule=useModule)
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
         self.spProxy = _ModuleProxy(searchPersonParser, defaultKeys=_def,
-                                    oldParsers=oldParsers, useModule=useModule)
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
         self.scProxy = _ModuleProxy(searchCharacterParser, defaultKeys=_def,
-                                    oldParsers=oldParsers, useModule=useModule)
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
         self.scompProxy = _ModuleProxy(searchCompanyParser, defaultKeys=_def,
-                                    oldParsers=oldParsers, useModule=useModule)
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
         self.mProxy = _ModuleProxy(movieParser, defaultKeys=_def,
-                                    oldParsers=oldParsers, useModule=useModule)
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
         self.pProxy = _ModuleProxy(personParser, defaultKeys=_def,
-                                    oldParsers=oldParsers, useModule=useModule)
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
         self.cProxy = _ModuleProxy(characterParser, defaultKeys=_def,
-                                    oldParsers=oldParsers, useModule=useModule)
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
         self.compProxy = _ModuleProxy(companyParser, defaultKeys=_def,
-                                    oldParsers=oldParsers, useModule=useModule)
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
 
     def _normalize_movieID(self, movieID):
         """Normalize the given movieID."""
@@ -362,6 +394,8 @@ class IMDbHTTPAccessSystem(IMDbBase):
             ton = ton.encode('utf-8')
         ##params = 'q=%s&%s=on&mx=%s' % (quote_plus(ton), kind, str(results))
         params = 's=%s;mx=%s;q=%s' % (kind, str(results), quote_plus(ton))
+        if kind == 'ep':
+            params = params.replace('s=ep;', 's=tt;ttype=ep;', 1)
         cont = self._retrieve(imdbURL_find % params)
         #print 'URL:', imdbURL_find % params
         if cont.find('more than 500 partial matches') == -1:
@@ -382,6 +416,13 @@ class IMDbHTTPAccessSystem(IMDbBase):
         cont = self._get_search_content('tt', title, results)
         return self.smProxy.search_movie_parser.parse(cont, results=results)['data']
 
+    def _search_episode(self, title, results):
+        t_dict = analyze_title(title)
+        if t_dict['kind'] == 'episode':
+            title = t_dict['title']
+        cont = self._get_search_content('ep', title, results)
+        return self.smProxy.search_movie_parser.parse(cont, results=results)['data']
+
     def get_movie_main(self, movieID):
         if not self.isThin:
             cont = self._retrieve(imdbURL_movie_main % movieID + 'combined')
