@@ -298,7 +298,19 @@ class HTMLMaindetailsParser(ParserBase):
 
 
 from movieParser import _manageRoles
-_reRoles = re.compile(r'<li>.*? \.\.\.\. (.*?)</li>', re.I)
+_reRoles = re.compile(r'(<li>.*? \.\.\.\. )(.*?)(</li>|<br>)',
+                        re.I | re.M | re.S)
+
+def build_date(date):
+    day = date.get('day')
+    year = date.get('year')
+    if day and year:
+        return "%s %s" % (day, year)
+    if day:
+        return day
+    if year:
+        return year
+    return ""
 
 class DOMHTMLMaindetailsParser(DOMParserBase):
     """Parser for the "categorized" (maindetails) page of a given person.
@@ -319,8 +331,7 @@ class DOMHTMLMaindetailsParser(DOMParserBase):
                             'year': "./a[starts-with(@href, " \
                                     "'/BornInYear?')]/text()"
                             },
-                        postprocess=lambda x: u"%s %s" % (x.get('day'),
-                                                        x.get('year'))),
+                        postprocess=lambda x: build_date(x)),
                     Attribute(key='birth notes',
                         path="./a[starts-with(@href, '/BornWhere?')]/text()")]
     _death_attrs = [Attribute(key='death date',
@@ -330,8 +341,7 @@ class DOMHTMLMaindetailsParser(DOMParserBase):
                             'year': "./a[starts-with(@href, " \
                                     "'/DiedInYear?')]/text()"
                             },
-                        postprocess=lambda x: u"%s %s" % (x.get('day'),
-                                                        x.get('year'))),
+                        postprocess=lambda x: build_date(x)),
                     Attribute(key='death notes',
                         path="./text()",
                         # TODO: check if this slicing is always correct
@@ -572,8 +582,7 @@ class DOMHTMLBioParser(DOMParserBase):
                             'year': "./a[starts-with(@href, " \
                                     "'/BornInYear?')]/text()"
                             },
-                        postprocess=lambda x: u"%s %s" % (x.get('day'),
-                                                        x.get('year'))),
+                        postprocess=lambda x: build_date(x)),
                     Attribute(key='birth notes',
                         path="./a[starts-with(@href, '/BornWhere?')]/text()")]
     _death_attrs = [Attribute(key='death date',
@@ -583,8 +592,7 @@ class DOMHTMLBioParser(DOMParserBase):
                             'year': "./a[starts-with(@href, " \
                                     "'/DiedInYear?')]/text()"
                             },
-                        postprocess=lambda x: u"%s %s" % (x.get('day'),
-                                                        x.get('year'))),
+                        postprocess=lambda x: build_date(x)),
                     Attribute(key='death notes',
                         path="./text()",
                         # TODO: check if this slicing is always correct
@@ -623,7 +631,8 @@ class DOMHTMLBioParser(DOMParserBase):
                                 'by': "./b/following-sibling::a/text()"
                                 },
                             postprocess=lambda x: "%s::%s" % \
-                                (x.get('by').strip(), x.get('bio').strip()))),
+                                (x.get('bio').strip(),
+                                (x.get('by') or u'').strip() or u'Anonymous'))),
             Extractor(label='spouse',
                         path="//div[h5='Spouse']/table/tr",
                         attrs=Attribute(key='spouse',
@@ -975,7 +984,7 @@ def _build_episode(link, title, minfo, role, roleA, roleAID):
                 if eps_data.get('year', '????') == '????':
                     syear = date.split()[-1]
                     if syear.isdigit():
-                        e['year'] = syear
+                        e['year'] = int(syear)
     return e
 
 
@@ -1023,7 +1032,8 @@ class DOMHTMLSeriesParser(DOMParserBase):
             link = self.xpath(dom, "//a/@href")[0]
             title = self.xpath(dom, "//a/text()")[0][1:-1]
             series = Movie(movieID=analyze_imdbid(link),
-                           data=analyze_title(title, canonical=1))
+                           data=analyze_title(title, canonical=1),
+                           accessSystem=self._as, modFunct=self._modFunct)
             nd[series] = []
             for episode in data[key]:
                 # XXX: should we create a copy of 'series', to avoid
