@@ -49,7 +49,7 @@ def analyze_imdbid(href):
     match = re_imdbid.search(href)
     if not match:
         return None
-    return match.group(2)
+    return str(match.group(2))
 
 
 _modify_keys = list(Movie.keys_tomodify_list) + list(Person.keys_tomodify_list)
@@ -201,9 +201,16 @@ def build_person(txt, personID=None, billingPos=None,
             roleID += [None] * (lrid - lr)
         elif lr < lrid:
             roleID = roleID[:lr]
+        for i, rid in enumerate(roleID):
+            if rid is not None:
+                roleID[i] = str(rid)
         if lr == 1:
             role = role[0]
             roleID = roleID[0]
+    elif roleID is not None:
+        roleID = str(roleID)
+    if personID is not None:
+        personID = str(personID)
     # XXX: return None if something strange is detected?
     return Person(name=name, personID=personID, currentRole=role,
                     roleID=roleID, notes=notes, billingPos=billingPos,
@@ -240,6 +247,8 @@ def build_movie(txt, movieID=None, roleID=None, status=None,
                 role = role[:nidx].rstrip()
     if title[-9:] == 'TV Series':
         title = title[:-9].rstrip()
+    elif title[-14:] == 'TV mini-series':
+        title = title[:-14] + ' (mini)'
     # Try to understand where the movie title ends.
     while True:
         if title[-1:] != ')':
@@ -285,9 +294,16 @@ def build_movie(txt, movieID=None, roleID=None, status=None,
             roleID += [None] * (lrid - lr)
         elif lr < lrid:
             roleID = roleID[:lr]
+        for i, rid in enumerate(roleID):
+            if rid is not None:
+                roleID[i] = str(rid)
         if lr == 1:
             role = role[0]
             roleID = roleID[0]
+    elif roleID is not None:
+        roleID = str(roleID)
+    if movieID is not None:
+        movieID = str(movieID)
     m = Movie(title=title, movieID=movieID, notes=notes, currentRole=role,
                 roleID=roleID, roleIsPerson=_parsingCharacter,
                 modFunct=modFunct, accessSystem=accessSystem)
@@ -584,7 +600,6 @@ class DOMParserBase(object):
     DOM/XPath version."""
     _defGetRefs = False
     _containsObjects = False
-    _fixRowSpans = False
 
     preprocessors = []
     extractors = []
@@ -599,25 +614,37 @@ class DOMParserBase(object):
         nrMods = len(useModule)
         _gotError = False
         for idx, mod in enumerate(useModule):
-            mod = mod.lower()
+            mod = mod.strip().lower()
             try:
                 if mod == 'lxml':
                     from lxmladapter import fromstring
                     from lxmladapter import tostring
-                    from lxmladapter import fix_rowspans
+                    from lxmladapter import setattribute
+                    from lxmladapter import getattribute
+                    from lxmladapter import getparent
+                    from lxmladapter import droptree
+                    from lxmladapter import clone
                     from lxmladapter import apply_xpath
                 elif mod == 'beautifulsoup':
                     from bsoupadapter import fromstring
                     from bsoupadapter import tostring
-                    from bsoupadapter import fix_rowspans
+                    from bsoupadapter import setattribute
+                    from bsoupadapter import getattribute
+                    from bsoupadapter import getparent
+                    from bsoupadapter import droptree
+                    from bsoupadapter import clone
                     from bsoupadapter import apply_xpath
                 else:
                     warnings.warn('unknown module "%s".' % mod)
                     continue
                 self.fromstring = fromstring
                 self.tostring = tostring
+                self.setattribute = setattribute
+                self.getattribute = getattribute
+                self.getparent = getparent
+                self.droptree = droptree
+                self.clone = clone
                 self.apply_xpath = apply_xpath
-                self.fix_rowspans = fix_rowspans
                 if _gotError:
                     warnings.warn('falling back to "%s".' % mod)
                 break
@@ -668,11 +695,13 @@ class DOMParserBase(object):
         ##html_string = subSGMLRefs(html_string)
         # Temporary fix: self.parse_dom must work even for empty strings.
         html_string = self.preprocess_string(html_string)
-        if self.getRefs:
-            self.gather_refs(html_string)
         html_string = html_string.strip()
         if html_string:
-            data = self.parse_dom(html_string)
+            dom = self.get_dom(html_string)
+            dom = self.preprocess_dom(dom)
+            if self.getRefs:
+                self.gather_refs(dom)
+            data = self.parse_dom(dom)
         else:
             data = {}
         data = self.postprocess_data(data)
@@ -687,9 +716,6 @@ class DOMParserBase(object):
     def xpath(self, element, xpath):
         return self.apply_xpath(element, xpath)
 
-    def _fix_rowspans(self, html_string):
-        return self.fix_rowspans(html_string)
-
     def preprocess_string(self, html_string):
         """Here we can modify the text, before it's parsed."""
         if len(html_string) == 0:
@@ -706,28 +732,31 @@ class DOMParserBase(object):
                 html_string = html_string.replace(src, sub)
             elif callable(src):
                 html_string = src(html_string)
-        if self._fixRowSpans:
-            html_string = self._fix_rowspans(html_string)
         ##print html_string.encode('utf8')
         return html_string
 
-    def gather_refs(self, html_string):
-        """Collect refs."""
-        # XXX: does this degrades performances?  Probably not at all.
+    def gather_refs(self, dom):
+        """Collect references."""
         grParser = GatherRefs(useModule=self._useModule)
         grParser._as = self._as
         grParser._modFunct = self._modFunct
-        refs = grParser.parse(html_string)
+        refs = grParser.parse_dom(dom)
+        refs = grParser.postprocess_data(refs)
         self._namesRefs = refs['names refs']
         self._titlesRefs = refs['titles refs']
         self._charactersRefs = refs['characters refs']
 
-    def parse_dom(self, html_string):
-        """Parse the given string according to the rules specified
+    def preprocess_dom(self, dom):
+        """Last chance to modify the dom, before the rules in self.extractors
+        are applied by the parse_dom method."""
+        return dom
+
+    def parse_dom(self, dom):
+        """Parse the given dom according to the rules specified
         in self.extractors."""
-        dom = self.get_dom(html_string)
         result = {}
         for extractor in self.extractors:
+            ##print extractor.label
             if extractor.group is None:
                 elements = [(extractor.label, element)
                             for element in self.xpath(dom, extractor.path)]
@@ -918,7 +947,7 @@ class GatherRefs(DOMParserBase):
             result[item] = {}
             for k, v in data.get(item, []):
                 if not v.endswith('/'): continue
-                imdbID = str(analyze_imdbid(v))
+                imdbID = analyze_imdbid(v)
                 if item == 'names refs':
                     obj = Person(personID=imdbID, name=k,
                                 accessSystem=self._as, modFunct=self._modFunct)
@@ -928,6 +957,8 @@ class GatherRefs(DOMParserBase):
                 else:
                     obj = Character(characterID=imdbID, name=k,
                                 accessSystem=self._as, modFunct=self._modFunct)
+                # XXX: companies aren't handled: are they ever found in text,
+                #      as links to their page?
                 result[item][k] = obj
         return result
 
