@@ -5,7 +5,7 @@ This module provides the functions used to parse the
 information about movies in a local installation of the
 IMDb database.
 
-Copyright 2004-2006 Davide Alberani <da@erlug.linux.it>
+Copyright 2004-2008 Davide Alberani <da@erlug.linux.it>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -133,7 +133,7 @@ def getPlot(movieID, plotIF, plotDF):
         if line.startswith('PL: '):
             plotltmp.append(line[4:])
         elif line.startswith('BY: '):
-            plotl.append('%s::%s' % (line[4:].strip(), ' '.join(plotltmp)))
+            plotl.append('%s::%s' % (' '.join(plotltmp), line[4:].strip()))
             plotltmp[:] = []
         elif line.startswith('MV: ') or not line: break
     dataf.close()
@@ -201,6 +201,23 @@ def getLiterature(movieID, indexF, dataF):
     return _parseColonList(movieID, indexF, dataF, 'MOVI: ', _lit)
 
 
+def getMPAA(movieID, indexF, dataF):
+    """Return MPAA reason information for a movie."""
+    try:
+        mpaa = _parseColonList(movieID, indexF, dataF, 'MV: ', {'RE': 'mpaa'})
+        if mpaa:
+            mpaa = mpaa.get('mpaa', u'')
+            mpaa = {'mpaa': u' '.join(mpaa)}
+        else:
+            mpaa = None
+        return mpaa
+    except IMDbDataAccessError:
+        import warnings
+        warnings.warn('MPAA info not accessible; please run the '
+                    'mpaa4local.py script.')
+        return {}
+
+
 _bus = {'BT': 'budget',
         'WG': 'weekend gross',
         'GR': 'gross',
