@@ -111,6 +111,9 @@ def _findBetween(s, begins, ends, beginindx=0, maxRes=None):
     return lres
 
 
+# Remove AKAs.
+_reAKAS = re.compile(r'aka <em.*?</td>', re.I | re.M)
+
 class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
     """The class used to access IMDb's data through the web for
     mobile terminals."""
@@ -190,13 +193,17 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
                 if mid and not mid[0].isdigit():
                     mid = re_imdbID.findall(mid[0])
             if not (mid and title): return res
+            if cont.find('<span class="tv-extra">TV mini-series</span>') != -1:
+                title += ' (mini)'
             res[:] = [(str(mid[0]), analyze_title(title, canonical=1))]
         else:
-            lis = _findBetween(cont, 'td valign="top">', ['</td>', '<small>'])
+            cont = _reAKAS.sub('</td>', cont)
+            lis = _findBetween(cont, 'td valign="top">', ['</td>', '</small>'])
             for li in lis:
                 imdbid = re_imdbID.findall(li)
                 mtitle = _unHtml(li)
                 if not (imdbid and mtitle): continue
+                mtitle = mtitle.replace('(TV mini-series)', '(mini)')
                 res.append((str(imdbid[0]), analyze_title(mtitle, canonical=1)))
         return res
 
@@ -206,6 +213,8 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         if not title:
             raise IMDbDataAccessError, 'unable to get movieID "%s"' % movieID
         title = _unHtml(title[0])
+        if cont.find('<span class="tv-extra">TV mini-series</span>') != -1:
+            title += ' (mini)'
         d = analyze_title(title, canonical=1)
         kind = d.get('kind')
         tv_series = _findBetween(cont, 'TV Series:</h5>', '</a>', maxRes=1)
@@ -429,10 +438,10 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             wbyidx = p.rfind(' Written by ')
             if wbyidx != -1:
                 plot[i] = '%s::%s' % \
-                    (p[wbyidx+12:].rstrip().replace('{','<').replace('}','>'),
-                    p[:wbyidx].rstrip())
+                        (p[:wbyidx].rstrip(),
+                    p[wbyidx+12:].rstrip().replace('{','<').replace('}','>'))
             else:
-                plot[i] = 'Anonymous::%s' % p
+                plot[i] = '%s::Anonymous' % p
         if plot: return {'data': {'plot': plot}}
         return {'data': {}}
 
@@ -665,8 +674,10 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
                 for bio in data:
                     byidx = bio.rfind('IMDb Mini Biography By')
                     if byidx != -1:
-                        bio = u'%s::%s' % (bio[byidx+23:].lstrip(),
-                                            bio[:byidx].rstrip())
+                        bioAuth = bio[:byidx].rstrip()
+                    else:
+                        bioAuth = 'Anonymous'
+                    bio = u'%s::%s' % (bioAuth, bio[byidx+23:].lstrip())
                     ndata.append(bio)
                 data[:] = ndata
                 if 'mini biography' in d:
