@@ -28,10 +28,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #        The code should be commented, rewritten and cleaned. :-)
 
 from types import UnicodeType
-
-from sqlobject.sqlbuilder import *
-
-from dbschema import *
+import warnings
 
 from imdb.parser.common.locsql import IMDbLocalAndSqlAccessSystem, \
                     scan_names, scan_titles, titleVariations, \
@@ -47,7 +44,6 @@ from imdb._exceptions import IMDbDataAccessError, IMDbError
 try:
     from imdb.parser.common.cutils import soundex
 except ImportError:
-    import warnings
     warnings.warn('Unable to import the cutils.soundex function.'
                     '  Searches of movie titles and person names will be'
                     ' a bit slower.')
@@ -131,6 +127,11 @@ def get_movie_data(movieID, kindDict, fromAka=0):
             mdict['series years'] = unicode(m.seriesYears)
     if mdict['imdbIndex'] is None: del mdict['imdbIndex']
     if mdict['year'] is None: del mdict['year']
+    else:
+        try:
+            mdict['year'] = int(mdict['year'])
+        except (TypeError, ValueError):
+            del mdict['year']
     if mdict['season'] is None: del mdict['season']
     else:
         try: mdict['season'] = int(mdict['season'])
@@ -156,12 +157,61 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
 
     accessSystem = 'sql'
 
-    def __init__(self, uri, adultSearch=1, *arguments, **keywords):
+    def __init__(self, uri, adultSearch=1, useORM=None, *arguments, **keywords):
         """Initialize the access system."""
         IMDbLocalAndSqlAccessSystem.__init__(self, *arguments, **keywords)
+        if useORM is None:
+            useORM = ('sqlobject', 'sqlalchemy')
+        if not isinstance(useORM, (tuple, list)):
+            if ',' in useORM:
+                useORM = useORM.split(',')
+            else:
+                useORM = [useORM]
+        self.useORM = useORM
+        nrMods = len(useORM)
+        _gotError = False
+        DB_TABLES = []
+        for idx, mod in enumerate(useORM):
+            mod = mod.strip().lower()
+            try:
+                if mod == 'sqlalchemy':
+                    from alchemyadapter import getDBTables, NotFoundError, \
+                                                setConnection, AND, OR, IN, \
+                                                ISNULL, toUTF8
+                elif mod == 'sqlobject':
+                    from objectadapter import getDBTables, NotFoundError, \
+                                                setConnection, AND, OR, IN, \
+                                                ISNULL, toUTF8
+                else:
+                    warnings.warn('unknown module "%s".' % mod)
+                    continue
+                # XXX: look ma'... black magic!  It's used to make
+                #      TableClasses and some functions accessible
+                #      through the whole module.
+                for k, v in [('NotFoundError', NotFoundError),
+                            ('AND', AND), ('OR', OR), ('IN', IN),
+                            ('ISNULL', ISNULL)]:
+                    globals()[k] = v
+                self.toUTF8 = toUTF8
+                DB_TABLES = getDBTables(uri)
+                for t in DB_TABLES:
+                    globals()[t._imdbpyName] = t
+                if _gotError:
+                    warnings.warn('falling back to "%s".' % mod)
+                break
+            except ImportError, e:
+                if idx+1 >= nrMods:
+                    raise IMDbError, 'unable to use any ORM in %s: %s' % (
+                                                    str(useORM), str(e))
+                else:
+                    warnings.warn('unable to use "%s": %s' % (mod, str(e)))
+                    _gotError = True
+                continue
+        else:
+            raise IMDbError, 'unable to use any ORM in %s' % str(useORM)
         # Set the connection to the database.
         try:
-            self._connection = setConnection(uri)
+            self._connection = setConnection(uri, DB_TABLES)
         except AssertionError, e:
             raise IMDbDataAccessError, \
                     'unable to connect to the database server; ' + \
@@ -213,7 +263,10 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         if val is None:
             return ISNULL(col)
         else:
-            return col == val.encode('utf_8')
+            if isinstance(val, (int, long)):
+                return col == val
+            else:
+                return col == self.toUTF8(val)
 
     def _getTitleID(self, title):
         """Given a long imdb canonical title, returns a movieID or
@@ -223,7 +276,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         if td['kind'] == 'episode':
             epof = td['episode of']
             seriesID = [s.id for s in Title.select(
-                        AND(Title.q.title == epof['title'].encode('utf_8'),
+                        AND(Title.q.title == self.toUTF8(epof['title']),
                             self._buildNULLCondition(Title.q.imdbIndex,
                                                     epof.get('imdbIndex')),
                            Title.q.kindID == self._kindRev[epof['kind']],
@@ -231,14 +284,14 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
                                                     epof.get('year'))))]
             if seriesID:
                 condition = AND(IN(Title.q.episodeOfID, seriesID),
-                                Title.q.title == td['title'].encode('utf_8'),
+                                Title.q.title == self.toUTF8(td['title']),
                                 self._buildNULLCondition(Title.q.imdbIndex,
                                                         td.get('imdbIndex')),
                                 Title.q.kindID == self._kindRev[td['kind']],
                                 self._buildNULLCondition(Title.q.productionYear,
                                                         td.get('year')))
         if condition is None:
-            condition = AND(Title.q.title == td['title'].encode('utf_8'),
+            condition = AND(Title.q.title == self.toUTF8(td['title']),
                             self._buildNULLCondition(Title.q.imdbIndex,
                                                     td.get('imdbIndex')),
                             Title.q.kindID == self._kindRev[td['kind']],
@@ -256,7 +309,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         """Given a long imdb canonical name, returns a personID or
         None if not found."""
         nd = analyze_name(name)
-        res = Name.select(AND(Name.q.name == nd['name'].encode('utf_8'),
+        res = Name.select(AND(Name.q.name == self.toUTF8(nd['name']),
                                 self._buildNULLCondition(Name.q.imdbIndex,
                                                         nd.get('imdbIndex'))))
         try:
@@ -305,7 +358,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         return None if it's unable to get the imdbID.
         """
         try: movie = Title.get(movieID)
-        except SQLObjectNotFound: return None
+        except NotFoundError: return None
         imdbID = movie.imdbID
         if imdbID is not None: return '%07d' % imdbID
         m_dict = get_movie_data(movie.id, self._kind)
@@ -327,7 +380,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         return None if it's unable to get the imdbID.
         """
         try: person = Name.get(personID)
-        except SQLObjectNotFound: return None
+        except NotFoundError: return None
         imdbID = person.imdbID
         if imdbID is not None: return '%07d' % imdbID
         n_dict = {'name': person.name, 'imdbIndex': person.imdbIndex}
@@ -344,7 +397,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         return None if it's unable to get the imdbID.
         """
         try: character = CharName.get(characterID)
-        except SQLObjectNotFound: return None
+        except NotFoundError: return None
         imdbID = character.imdbID
         if imdbID is not None: return '%07d' % imdbID
         n_dict = {'name': character.name, 'imdbIndex': character.imdbIndex}
@@ -361,7 +414,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         return None if it's unable to get the imdbID.
         """
         try: company = CompanyName.get(companyID)
-        except SQLObjectNotFound: return None
+        except NotFoundError: return None
         imdbID = company.imdbID
         if imdbID is not None: return '%07d' % imdbID
         n_dict = {'name': company.name, 'country': company.countryCode}
@@ -378,7 +431,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         shown in the results of a search."""
         self.doAdult = doAdult
 
-    def _search_movie(self, title, results):
+    def _search_movie(self, title, results, _episodes=False):
         title = title.strip()
         if not title: return []
         title_dict = analyze_title(title, canonical=1)
@@ -387,11 +440,15 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         episodeOf = title_dict.get('episode of')
 
         if not episodeOf:
-            s_title_split = s_title.split(', ')
-            if len(s_title_split) >1 and s_title_split[-1].lower() in _articles:
-                s_title_rebuilt = ', '.join(s_title_split[:-1])
-                if s_title_rebuilt: s_title = s_title_rebuilt
+            if not _episodes:
+                s_title_split = s_title.split(', ')
+                if len(s_title_split) > 1 and \
+                        s_title_split[-1].lower() in _articles:
+                    s_title_rebuilt = ', '.join(s_title_split[:-1])
+                    if s_title_rebuilt:
+                        s_title = s_title_rebuilt
         else:
+            _episodes = False
             s_title = normalizeTitle(s_title)
         if isinstance(s_title, UnicodeType):
             s_title = s_title.encode('ascii', 'ignore')
@@ -401,7 +458,12 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         # XXX: improve the search restricting the kindID if the
         #      "kind" of the input differs from "movie"?
         condition = conditionAka = None
-        if title_dict['kind'] == 'episode' and episodeOf is not None:
+        if _episodes:
+            condition = AND(Title.q.phoneticCode == soundexCode,
+                            Title.q.kindID == self._kindRev['episode'])
+            conditionAka = AND(AkaTitle.q.phoneticCode == soundexCode,
+                            AkaTitle.q.kindID == self._kindRev['episode'])
+        elif title_dict['kind'] == 'episode' and episodeOf is not None:
             series_title = build_title(episodeOf, canonical=1)
             # XXX: is it safe to get "results" results?
             #      Too many?  Too few?
@@ -434,15 +496,19 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
 
         # Up to 3 variations of the title are searched, plus the
         # long imdb canonical title, if provided.
-        title1, title2, title3 = titleVariations(title)
-
+        if not _episodes:
+            title1, title2, title3 = titleVariations(title)
+        else:
+            title1 = title
+            title2 = ''
+            title3 = ''
         try:
             qr = [(q.id, get_movie_data(q.id, self._kind))
                     for q in Title.select(condition)]
             q2 = [(q.movieID, get_movie_data(q.id, self._kind, fromAka=1))
                     for q in AkaTitle.select(conditionAka)]
             qr += q2
-        except SQLObjectNotFound, e:
+        except NotFoundError, e:
             raise IMDbDataAccessError, \
                     'unable to search the database: "%s"' % str(e)
 
@@ -450,6 +516,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         if not self.doAdult: resultsST = 0
         res = scan_titles(qr, title1, title2, title3, resultsST,
                             searchingEpisode=episodeOf is not None,
+                            onlyEpisodes=_episodes,
                             ro_thresold=0.0)
         if self.doAdult and results > 0: res[:] = res[:results]
         res[:] = [x[1] for x in res]
@@ -466,12 +533,15 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
             if results > 0: res[:] = res[:results]
         return res
 
+    def _search_episode(self, title, results):
+        return self._search_movie(title, results, _episodes=True)
+
     def get_movie_main(self, movieID):
         # Every movie information is retrieved from here.
         infosets = self.get_movie_infoset()
         try:
             res = get_movie_data(movieID, self._kind)
-        except SQLObjectNotFound, e:
+        except NotFoundError, e:
             raise IMDbDataAccessError, \
                     'unable to get movieID "%s": "%s"' % (movieID, str(e))
         if not res:
@@ -594,16 +664,16 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         res = _reGroupDict(res, self._moviesubs)
         # Do some transformation to preserve consistency with other
         # data access systems.
-        if res.has_key('plot'):
-            nl = []
-            for i in res['plot']:
-                if i[-1] == ')':
-                    sauth = i.rfind('::(author: ')
-                    if sauth != -1:
-                        nl.append(i[sauth+11:-1] + '::' + i[:sauth])
-                    else: nl.append(i)
-                else: nl.append(i)
-            res['plot'][:] = nl
+        #if res.has_key('plot'):
+        #    nl = []
+        #    for i in res['plot']:
+        #        if i[-1] == ')':
+        #            sauth = i.rfind('::(author: ')
+        #            if sauth != -1:
+        #                nl.append(i[sauth+11:-1] + '::' + i[:sauth])
+        #            else: nl.append(i)
+        #        else: nl.append(i)
+        #    res['plot'][:] = nl
         # Other transformations.
         if res.has_key('runtimes') and len(res['runtimes']) > 0:
             rt = res['runtimes'][0]
@@ -612,8 +682,8 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
                 res['runtimes'][0] = re_episodes.sub('', rt)
                 if res['runtimes'][0][-2:] == '::':
                     res['runtimes'][0] = res['runtimes'][0][:-2]
-        if res.has_key('year'):
-            res['year'] = res['year']
+        #if res.has_key('year'):
+        #    res['year'] = res['year']
         if res.has_key('votes'):
             res['votes'] = int(res['votes'][0])
         if res.has_key('rating'):
@@ -680,7 +750,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
                     for q in Name.select(condition)]
             qr += [(q.personID, {'name': q.name, 'imdbIndex': q.imdbIndex})
                     for q in AkaName.select(conditionAka)]
-        except SQLObjectNotFound, e:
+        except NotFoundError, e:
             raise IMDbDataAccessError, \
                     'unable to search the database: "%s"' % str(e)
 
@@ -700,7 +770,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         infosets = self.get_person_infoset()
         try:
             p = Name.get(personID)
-        except SQLObjectNotFound, e:
+        except NotFoundError, e:
             raise IMDbDataAccessError, \
                     'unable to get personID "%s": "%s"' % (personID, str(e))
         res = {'name': p.name, 'imdbIndex': p.imdbIndex}
@@ -783,16 +853,16 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
                         'birth name', 'height'):
             if res.has_key(key):
                 res[key] = res[key][0]
-        if res.has_key('mini biography'):
-            nl = []
-            for i in res['mini biography']:
-                if i[-1] == ')':
-                    sauth = i.rfind('::(author: ')
-                    if sauth != -1:
-                        nl.append(i[sauth+11:-1] + '::' + i[:sauth])
-                    else: nl.append(i)
-                else: nl.append(i)
-            res['mini biography'][:] = nl
+        #if res.has_key('mini biography'):
+        #    nl = []
+        #    for i in res['mini biography']:
+        #        if i[-1] == ')':
+        #            sauth = i.rfind('::(author: ')
+        #            if sauth != -1:
+        #                nl.append(i[sauth+11:-1] + '::' + i[:sauth])
+        #            else: nl.append(i)
+        #        else: nl.append(i)
+        #    res['mini biography'][:] = nl
         if res.has_key('guest'):
             res['notable tv guest appearances'] = res['guest']
             del res['guest']
@@ -850,7 +920,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         try:
             qr = [(q.id, {'name': q.name, 'imdbIndex': q.imdbIndex})
                     for q in CharName.select(condition)]
-        except SQLObjectNotFound, e:
+        except NotFoundError, e:
             raise IMDbDataAccessError, \
                     'unable to search the database: "%s"' % str(e)
         res = scan_names(qr, s_name, name2, '', results,
@@ -870,7 +940,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         infosets = self.get_character_infoset()
         try:
             c = CharName.get(characterID)
-        except SQLObjectNotFound, e:
+        except NotFoundError, e:
             raise IMDbDataAccessError, \
                     'unable to get characterID "%s": "%s"' % (characterID, e)
         res = {'name': c.name, 'imdbIndex': c.imdbIndex}
@@ -925,7 +995,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         try:
             qr = [(q.id, {'name': q.name, 'country': q.countryCode})
                     for q in CompanyName.select(condition)]
-        except SQLObjectNotFound, e:
+        except NotFoundError, e:
             raise IMDbDataAccessError, \
                     'unable to search the database: "%s"' % str(e)
         qr[:] = [(x[0], build_company_name(x[1])) for x in qr]
@@ -946,7 +1016,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         infosets = self.get_company_infoset()
         try:
             c = CompanyName.get(companyID)
-        except SQLObjectNotFound, e:
+        except NotFoundError, e:
             raise IMDbDataAccessError, \
                     'unable to get companyID "%s": "%s"' % (companyID, e)
         res = {'name': c.name, 'country': c.countryCode}
