@@ -300,13 +300,19 @@ def analyze_title(title, canonical=None,
                                     episode_or_year[1:2] != '#':
                         oad = episode_or_year
                         if oad[1:5] and oad[5:6] == '-':
-                            ep_year = oad[1:5]
+                            try:
+                                ep_year = int(oad[1:5])
+                            except (TypeError, ValueError):
+                                pass
                 if not oad and not sen and episode_or_year.startswith('(#'):
                     sen = episode_or_year
         elif episode_or_year.startswith('Episode dated'):
             oad = episode_or_year[14:]
             if oad[-4:].isdigit():
-                ep_year = oad[-4:]
+                try:
+                    ep_year = int(oad[-4:])
+                except (TypeError, ValueError):
+                    pass
         episode_d = analyze_title(episode_or_year, canonical=canonicalEpisode)
         episode_d['kind'] = u'episode'
         episode_d['episode of'] = series_d
@@ -373,12 +379,14 @@ def analyze_title(title, canonical=None,
     result['title'] = title
     result['kind'] = kind or u'movie'
     if year and year != '????':
-        result['year'] = year
+        try:
+            result['year'] = int(year)
+        except (TypeError, ValueError):
+            pass
     if imdbIndex:
         result['imdbIndex'] = imdbIndex
     if isinstance(_emptyString, str):
         result['kind'] = str(kind or 'movie')
-        if year and year != '????': result['year'] = str(year)
     return result
 
 
@@ -438,7 +446,7 @@ def build_title(title_dict, canonical=None, canonicalSeries=0,
         ep_title = ep_dict['title']
         if not ptdf:
             doYear = 1
-            ep_dict['year'] = title_dict.get('year') or '????'
+            ep_dict['year'] = title_dict.get('year', '????')
             if ep_title[0:1] == '(' and ep_title[-1:] == ')' and \
                     ep_title[1:5].isdigit():
                 ep_dict['title'] = _convertTime(ep_title, fromPTDFtoWEB=1,
@@ -494,6 +502,57 @@ def build_title(title_dict, canonical=None, canonicalSeries=0,
     return title
 
 
+def split_company_name_notes(name):
+    """Return two strings, the first representing the company name,
+    and the other representing the (optional) notes."""
+    name = name.strip()
+    notes = u''
+    if name.endswith(')'):
+        fpidx = name.find('(')
+        if fpidx != -1:
+            notes = name[fpidx:]
+            name = name[:fpidx].rstrip()
+    return name, notes
+
+
+def analyze_company_name(name, stripNotes=False):
+    """Return a dictionary with the name and the optional 'country'
+    keys, from the given string.
+    If stripNotes is true, tries to not consider optional notes.
+
+    raise an IMDbParserError exception if the name is not valid.
+    """
+    if stripNotes:
+        name = split_company_name_notes(name)[0]
+    o_name = name
+    name = name.strip()
+    country = None
+    if name.endswith(']'):
+        idx = name.rfind('[')
+        if idx != -1:
+            country = name[idx:]
+            name = name[:idx].rstrip()
+    if not name:
+        raise IMDbParserError, 'invalid name: "%s"' % o_name
+    result = {'name': name}
+    if country:
+        result['country'] = country
+    return result
+
+
+def build_company_name(name_dict, _emptyString=u''):
+    """Given a dictionary that represents a "long" IMDb company name,
+    return a string.
+    """
+    name = name_dict.get('name')
+    if not name:
+        return _emptyString
+    country = name_dict.get('country')
+    if country is not None:
+        name += ' %s' % country
+    return name
+
+
 class _LastC:
     """Size matters."""
     def __cmp__(self, other):
@@ -525,10 +584,16 @@ def cmpMovies(m1, m2):
                 return 1
             elif m1p > m2p:
                 return -1
-    if m1e is None: m1y = int(m1.get('year', 0))
-    else: m1y = int(m1e.get('year', 0))
-    if m2e is None: m2y = int(m2.get('year', 0))
-    else: m2y = int(m2e.get('year', 0))
+    try:
+        if m1e is None: m1y = int(m1.get('year', 0))
+        else: m1y = int(m1e.get('year', 0))
+    except ValueError:
+        m1y = 0
+    try:
+        if m2e is None: m2y = int(m2.get('year', 0))
+        else: m2y = int(m2e.get('year', 0))
+    except ValueError:
+        m2y = 0
     if m1y > m2y: return -1
     if m1y < m2y: return 1
     # Ok, these movies have the same production year...
@@ -568,6 +633,22 @@ def cmpPeople(p1, p2):
     return 0
 
 
+def cmpCompanies(p1, p2):
+    """Compare two companies."""
+    p1n = p1.get('long imdb name', _last)
+    p2n = p2.get('long imdb name', _last)
+    if p1n is _last and p2n is _last:
+        p1n = p1.get('name', _last)
+        p2n = p2.get('name', _last)
+    if p1n > p2n: return 1
+    if p1n < p2n: return -1
+    p1i = p1.get('country', _last)
+    p2i = p2.get('country', _last)
+    if p1i > p2i: return 1
+    if p1i < p2i: return -1
+    return 0
+
+
 # References to titles, names and characters.
 # XXX: find better regexp!
 re_titleRef = re.compile(r'_(.+?(?: \([0-9\?]{4}(?:/[IVXLCDM]+)?\))?(?: \(mini\)| \(TV\)| \(V\)| \(VG\))?)_ \(qv\)')
@@ -874,11 +955,11 @@ class _Container(object):
             self.data = data
 
     def getID(self):
-        """Return movieID, personID or characterID."""
+        """Return movieID, personID, characterID or companyID."""
         raise NotImplementedError, 'override this method'
 
     def __cmp__(self, other):
-        """Compare two Movie, Person or Character objects."""
+        """Compare two Movie, Person, Character or Company objects."""
         # XXX: raise an exception?
         if self.cmpFunct is None: return -1
         if not isinstance(other, self.__class__): return -1
@@ -893,7 +974,8 @@ class _Container(object):
             acs = self.accessSystem
             if acs in ('mobile', 'httpThin'):
                 acs = 'http'
-            s4h = '%s:%s' % (acs, theID)
+            # There must be some indication of the kind of the object, too.
+            s4h = '%s:%s[%s]' % (self.__class__.__name__, theID, acs)
         else:
             s4h = repr(self)
         return hash(s4h)
